using Microsoft.AspNetCore.Mvc;
using MiniExcelLibs;
using WCS.BLL.HardWare;
using WCS.BLL.Manager;
using WCS.BLL.Services.IService;
using WCS.BLL.Services.Service;
using WCS.Model;
using WCS.Model.ApiModel.MatBaseInfo;
using WCS.Model.ApiModel.OutStore;

namespace WebApi.Controllers
{
    [ApiController]
    [Route("[controller]")]
    public class OutstoreController : ControllerBase
    {
        private readonly IOutstoreService _outstoreService;
        private readonly IGenerateService _generateService;

        public OutstoreController(IOutstoreService outstoreService, IGenerateService generateService)
        {
            _outstoreService = outstoreService;
            _generateService = generateService;
        }


        /// <summary>
        /// ģУͱȶ
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [Route("importMat")]
        [HttpPost(Name = "importMat")]
        public async Task<ResponseBase> importMat([FromForm] IFormFile excelFile, [FromForm] string userName, [FromForm] string deviceType)
        {
            try
            {
                //ļУ
                if (excelFile == null || excelFile.Length == 0)
                {
                    return new ResponseCommon()
                    {
                        Code = 201,
                        Message = "ʧ:ļЧݣ"
                    };
                }
                //ȡе
                using (var stream = new MemoryStream())
                {
                    await excelFile.CopyToAsync(stream);
                    stream.Position = 0;
                    var list = MiniExcelLibs.MiniExcel.Query<OutImportMatModel>(stream, "⵼ģ", ExcelType.XLSX).ToList();
                    return await _outstoreService.importMat(list);
                }
            }
            catch (Exception ex)
            {
                return new ResponseBase()
                {
                    Code = 201,
                    Message = "ʧܣ" + ex.Message,
                };
            }
        }



        /// <summary>
        /// ϱͬ
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [Route("sysOutOrderByMatCode")]
        [HttpPost(Name = "sysOutOrderByMatCode")]
        public async Task<ResponseBase> sysOutOrderByMatCode(SysOutOrderByMatCodeRequest request)
        {
            try
            {
                if (string.IsNullOrEmpty(request.OrderNumber))
                {
                    request.OrderNumber = await _generateService.generateOutOrderNumber();
                }
                return await _outstoreService.SysOutOrderByMatCode(request);
            }
            catch (Exception ex)
            {
                return new ResponseBase()
                {
                    Code = 300,
                    Message = "ͬʧܣ" + ex.Message,
                };
            }
        }


        /// <summary>
        /// ϸͬ
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [Route("sysOutOrderByMatSn")]
        [HttpPost(Name = "sysOutOrderByMatSn")]
        public async Task<ResponseBase> sysOutOrderByMatSn(SysOutOrderByMatSnRequest request)
        {
            try
            {
                if (string.IsNullOrEmpty(request.OrderNumber))
                {
                    if (request.IsMXPD)
                    {
                         request.OrderNumber = await _generateService.generateMXPDOutOrderNumber();
                    }
                    else
                        request.OrderNumber = await _generateService.generateOutOrderNumber();
                }
                return await _outstoreService.SysOutOrderByMatSn(request);
            }
            catch (Exception ex)
            {
                return new ResponseBase()
                {
                    Code = 300,
                    Message = "ͬʧܣ" + ex.Message,
                };
            }
        }


        /// <summary>
        /// ѯصǰⵥб
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [Route("getOutOrderList")]
        [HttpPost(Name = "getOutOrderList")]
        public async Task<ResponseBase> getOutOrderList(GetOutOrderListRequest request)
        {
            try
            {
                return await _outstoreService.GetOutOrderList(request);
            }
            catch (Exception ex)
            {
                return new ResponseBase()
                {
                    Code = 300,
                    Message = "ѯʧܣ" + ex.Message,
                };
            }
        }

        /// <summary>
        /// ״̬ѯⵥ
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [Route("getOutOrderListByStatus")]
        [HttpPost(Name = "getOutOrderListByStatus")]
        public async Task<ResponseBase> getOutOrderListByStatus(GetOutOrderListByStatusRequest request)
        {
            try
            {
                return await _outstoreService.GetOutOrderListByStatus(request);
            }
            catch (Exception ex)
            {
                return new ResponseBase()
                {
                    Code = 300,
                    Message = "ѯʧܣ" + ex.Message,
                };
            }
        }

        /// <summary>
        /// ȡⵥĳϸ
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [Route("getOutOrderDetail")]
        [HttpPost(Name = "getOutOrderDetail")]
        public async Task<ResponseBase> getOutOrderDetail(GetOutOrderDetailRequest request)
        {
            try
            {
                return await _outstoreService.GetOutOrderDetail(request);
            }
            catch (Exception ex)
            {
                return new ResponseBase()
                {
                    Code = 300,
                    Message = "ѯʧܣ" + ex.Message,
                };
            }
        }

        /// <summary>
        /// ȡⵥϸ
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [Route("getOutOrderMatDetail")]
        [HttpPost(Name = "getOutOrderMatDetail")]
        public async Task<ResponseBase> getOutOrderMatDetail(GetOutOrderDetailRequest request)
        {
            try
            {
                return await _outstoreService.GetOutOrderMatDetail(request);
            }
            catch (Exception ex)
            {
                return new ResponseBase()
                {
                    Code = 300,
                    Message = "ѯʧܣ" + ex.Message,
                };
            }
        }


        /// <summary>
        /// ͨⵥݺſʼ
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [Route("goInOutstore")]
        [HttpPost(Name = "goInOutstore")]
        public async Task<ResponseBase> goInOutstore(GetOutOrderDetailRequest request)
        {
            try
            {
                return await _outstoreService.GoInOutstore(request);
            }
            catch (Exception ex)
            {
                return new ResponseBase()
                {
                    Code = 300,
                    Message = "ģʽʧܣ" + ex.Message,
                };
            }
        }

        /// <summary>
        /// ͨⵥݺ˳/
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [Route("goOutOutstore")]
        [HttpPost(Name = "goOutOutstore")]
        public async Task<ResponseBase> goOutOutstore(GetOutOrderDetailRequest request)
        {
            try
            {
                return await _outstoreService.GoOutOutstore(request);
            }
            catch (Exception ex)
            {
                return new ResponseBase()
                {
                    Code = 300,
                    Message = "ѯʧܣ" + ex.Message,
                };
            }
        }


        /// <summary>
        /// ƿʼ
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [Route("singleLightGoInOutstore")]
        [HttpPost(Name = "singleLightGoInOutstore")]
        public async Task<ResponseBase> singleLightGoInOutstore(GetOutOrderDetailRequest request)
        {
            //TODO:
            try
            {
                return await _outstoreService.GoInOutstoreSingle(request);
            }
            catch (Exception ex)
            {
                return new ResponseBase()
                {
                    Code = 300,
                    Message = "ģʽʧܣ" + ex.Message,
                };
            }
        }

        /// <summary>
        /// ƽ
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [Route("singleLightGoOutOutstore")]
        [HttpPost(Name = "singleLightGoOutOutstore")]
        public async Task<ResponseBase> singleLightGoOutOutstore(GetOutOrderDetailRequest request)
        {
            //TODO
            try
            {
                return await _outstoreService.GoOutOutstoreSingle(request);
            }
            catch (Exception ex)
            {
                return new ResponseBase()
                {
                    Code = 300,
                    Message = "ѯʧܣ" + ex.Message,
                };
            }
        }

        /// <summary>
        /// ȷϳ
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [Route("singleLightConfirmOutstore")]
        [HttpPost(Name = "singleLightConfirmOutstore")]
        public async Task<ResponseBase> singleLightConfirmOutstore(OutOrderMatDetailModelSingle request)
        {
            //TODO
            try
            {
                return await _outstoreService.SingleLightConfirmOutstore(request);
            }
            catch (Exception ex)
            {
                return new ResponseBase()
                {
                    Code = 300,
                    Message = "ѯʧܣ" + ex.Message,
                };
            }
        }

    }
}
