﻿using HandyControl.Controls;
using MiniExcelLibs;
using Ping9719.WpfEx.Mvvm;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Input;
using 货架标准上位机.Views.Controls;
using 货架标准上位机.Api;
using WCS.Model;
using WCS.Model.ApiModel.Home;
using WCS.Model.ApiModel.StoreInfo;
using WCS.BLL.DbModels;
using WCS.Model.ApiModel.MatBaseInfo;
using WCS.Model.ApiModel.User;
using WCS.Model.ApiModel;
using Newtonsoft.Json.Bson;
using System.Windows;

namespace 货架标准上位机.ViewModel
{
    public class ModuleInfoViewModel : BindableBase
    {
        public ModuleInfoViewModel()
        {

        }

        #region Property
        private List<ModuleInfoModel> dataGridItemSource;
        public List<ModuleInfoModel> DataGridItemSource
        {
            get { return dataGridItemSource; }
            set
            {
                SetProperty(ref dataGridItemSource, value);
            }
        }

        private ModuleInfoModel selectedataGridItem;
        public ModuleInfoModel SelectedataGridItem
        {
            get { return selectedataGridItem; }
            set
            {
                SetProperty(ref selectedataGridItem, value);
            }
        }


        private string shelfCode;
        public string ShelfCode
        {
            get { return shelfCode; }
            set
            {
                SetProperty(ref shelfCode, value);
            }
        }

        private string moduleCode;

        public string ModuleCode
        {
            get => moduleCode;
            set { SetProperty(ref moduleCode, value); }
        }

        #endregion

        #region Command
        public ICommand BtnResetCommand { get => new DelegateCommand(BtnReset); }
        public void BtnReset()
        {
            ModuleCode = string.Empty;
            ShelfCode = string.Empty;
        }

        public ICommand BtnSearchCommand { get => new DelegateCommand(BtnSearchReset); }
        public void BtnSearchReset()
        {
            BtnSearch(true);
        }

        public void BtnSearch(bool IsPageReset = true)
        {
            if (CurrentPage == 0 || IsPageReset)
            {
                CurrentPage = 1;
                return;
            }
            #region 调用接口获取数据
            var dia = Dialog.Show(new TextDialog());
            try
            {
                var body = new GetModulesRequest()
                {
                    ShelfCode = ShelfCode,
                    ModuleCode = ModuleCode,
                    UserName = LocalStatic.CurrentUser,
                    DeviceType = LocalFile.Config.DeviceType,
                    PageNumber = CurrentPage,
                    PageSize = PageSize,
                };
                var Result = ApiHelp.GetDataFromHttp<PageQueryResponse<ModuleInfoModel>>(LocalFile.Config.ApiIpHost + "storeInfo/getModules", body, "POST");
                if (Result != null && Result.Data != null && Result.Data.Lists != null)
                {
                    DataGridItemSource = Result.Data.Lists;
                    MaxPage = Result.Data.MaxPage;
                    TotalCount = Result.Data.TotalCount;
                }
            }
            catch (Exception ex)
            {
                Growl.Error("加载数据失败：" + ex.Message);
            }
            finally
            {
                dia.Close();
            }
            #endregion
        }

        public ICommand BtnDeleteCommand { get => new DelegateCommand(BtnDelete); }
        public void BtnDelete()
        {
            //查询勾选的第一个数据
            var shelfInfo = DataGridItemSource?.Where(t => t.IsSelected == true).FirstOrDefault();
            if (shelfInfo == null)
            {
                Growl.Warning("请选择需要删除的数据！");
            }
            else
            {
                var body = new AddShelfInfoRequest<ShelfInfoModel>()
                {
                    UserName = LocalStatic.CurrentUser,
                    DeviceType = LocalFile.Config.DeviceType,
                    //ShelfInfo = shelfInfo,
                    AddOrUpdate = AddOrUpdate.Delete
                };
                var Result = ApiHelp.GetDataFromHttp<ResponseBase<object>>(LocalFile.Config.ApiIpHost + "storeInfo/addOrUpdateShelfInfo", body, "POST");
                if (Result != null && Result.Code == 200)
                {
                    Growl.Success("删除成功！");
                    CurrentPage = 1;
                }
                else
                {
                    Growl.Error($"{Result?.Message?.ToString()}");
                }
            }
        }

        public ICommand DisableCommand { get => new DelegateCommand<ModuleInfoModel>(Disable); }
        public void Disable(ModuleInfoModel module)
        {
            if (module.IsEnable != true)
            {
                Growl.Warning("库位未被启用！");
                return;
            }
            var result = HandyControl.Controls.MessageBox.Show("模组禁用会影响正常流程，请确认是否屏蔽？"
                , "提示", MessageBoxButton.YesNo);
            if (result == MessageBoxResult.Yes)
            {
                #region 调用接口 禁用
                try
                {
                    var body = new DisableOrEnableModuleRequest()
                    {
                        ModuleId = module.Id,
                        ModuleCode = module.ModuleCode,
                        DisableOrEnable = DisableOrEnableEnum.Disable,
                        UserName = LocalStatic.CurrentUser,
                        DeviceType = LocalFile.Config.DeviceType,
                    };
                    var Result = ApiHelp.GetDataFromHttp<ResponseCommon>(LocalFile.Config.ApiIpHost + "storeInfo/disableOrEnableModule", body, "POST");
                    if (Result != null && Result.Code == 200)
                    {
                        Growl.Success("禁用成功");
                        BtnSearch();
                    }
                    else if (Result != null)
                    {
                        Growl.Warning(Result.Message);
                    }
                    else
                    {
                        Growl.Warning("操作失败：请重试！");
                    }
                }
                catch (Exception ex)
                {
                    Growl.Error("操作失败：" + ex.Message);
                }
                #endregion
            }
            else
            {
                return;
            }
        }

        public ICommand EnableCommand { get => new DelegateCommand<ModuleInfoModel>(Enable); }
        public void Enable(ModuleInfoModel module)
        {
            if (module.IsEnable == true)
            {
                Growl.Warning("库位未被禁用！");
                return;
            }
            #region 调用接口 临时禁用库位 删除库存数据
            try
            {
                var body = new DisableOrEnableModuleRequest()
                {
                    ModuleId = module.Id,
                    ModuleCode = module.ModuleCode,
                    DisableOrEnable = DisableOrEnableEnum.Enable,
                    UserName = LocalStatic.CurrentUser,
                    DeviceType = LocalFile.Config.DeviceType,
                };
                var Result = ApiHelp.GetDataFromHttp<ResponseCommon>(LocalFile.Config.ApiIpHost + "storeInfo/disableOrEnableModule", body, "POST");
                if (Result != null && Result.Code == 200)
                {
                    Growl.Success("操作成功！");
                    BtnSearch();
                }
                else if (Result != null)
                {
                    Growl.Warning(Result.Message);
                }
                else
                {
                    Growl.Warning("操作失败：请重试！");
                }
            }
            catch (Exception ex)
            {
                Growl.Error("操作失败：" + ex.Message);
            }
            #endregion
        }
        #endregion

        #region PageOperation 分页操作
        private int currentPage;
        public int CurrentPage
        {
            get { return currentPage; }
            set
            {
                SetProperty(ref currentPage, value);
                BtnSearch(false);
            }
        }

        private int maxPage;
        public int MaxPage
        {
            get { return maxPage; }
            set { SetProperty(ref maxPage, value); }
        }

        //总数量
        private int totalCount;
        public int TotalCount
        {
            get { return totalCount; }
            set { SetProperty(ref totalCount, value); }
        }

        private int pageSize = 10;
        public int PageSize
        {
            get => pageSize;
            set { SetProperty(ref pageSize, value); }
        }


        public ICommand BtnFirstPageCommand { get => new DelegateCommand(BtnFirstPage); }
        public void BtnFirstPage()
        {
            CurrentPage = 1;
        }

        public ICommand BtnPrePageCommand { get => new DelegateCommand(BtnPrePage); }
        public void BtnPrePage()
        {
            if (CurrentPage > 1)
            {
                CurrentPage--;
            }
        }

        public ICommand BtnNextPageCommand { get => new DelegateCommand(BtnNextPage); }
        public void BtnNextPage()
        {
            if (CurrentPage < MaxPage)
            {
                CurrentPage++;
            }
        }

        public ICommand BtnLastPageCommand { get => new DelegateCommand(BtnLastPage); }
        public void BtnLastPage()
        {
            if (CurrentPage != MaxPage)
            {
                CurrentPage = MaxPage;
            }
        }
        #endregion

    }
}
